import { 
	EVENT_FIELD_INPUT,
	EVENTHANDLE_INPUT_TIMEOUT 
} from "../Constants";
import { noValue } from "@default-js/defaultjs-common-utils/src/ValueHelper";
import { toTimeoutHandle } from "../utils/EventHelper";
import Wrapper from "./Wrapper";

const INPUTSELECTOR = 'input:not([type="file"],[type="radio"],[type="checkbox"],[type="button"],[type="submit"],[type="reset"]),input:not([type]), textarea';

const DEFAULTTYPE = "text";

const text = (input) => {
	return 	{
		accept: (value) => {
			return typeof value === "string"; 
		},
		getValue: () => {
			return input.value;
		},
		setValue: (value) => {
			return input.value = value; 
		},
		normalize: (value) => {
			if (value) {
				value = value.trim();
				return value.length > 0 ? value : null;
			}

			return null;
		}
	};
};
const number = (input) =>{
	return {
		accept: (value) => {
			return typeof value === "number";
		},
		getValue: () => {
			return input.valueAsNumber;
		},
		setValue: (value) =>{
			input.valueAsNumber = value;
		},
		normalize: (value) => {
			if (!noValue(value) && !Number.isNaN(value)) return value;

			return null;
		},
	};
};
const date = (input) => {
	return {
		accept: (value) => {
			return value instanceof Date;
		},
		getValue: () => {
			return input.valueAsDate;
		},
		setValue: (value) => {
			input.valueAsDate = value;
		},
		normalize: (value) => {
			if (value) return value;

			return null;
		},
	};
};
const TYPES = { text, number, date, time: date, range:number };

export default class Text extends Wrapper {
	static findInput(field) {
		return field.find(INPUTSELECTOR).first();
	}

	constructor(field, input) {
		super(field, input);
	}

	init() {
		const { field, input } = this;
		const type = (field.attr("input-type") || input.attr("type") || DEFAULTTYPE).trim().toLowerCase();
		this.type = (TYPES[type] || TYPES[DEFAULTTYPE])(input);
		input.on(
			"input",
			toTimeoutHandle(
				() => {
					field.trigger(EVENT_FIELD_INPUT, this.normalizeValue(this.value));
				},
				false,
				true,
				EVENTHANDLE_INPUT_TIMEOUT
			),
		);

		field.trigger(EVENT_FIELD_INPUT, this.normalizeValue(this.value));
	}

	acceptValue(value) {
		if (value == null || typeof value === "undefined") return true;

		return this.type.accept(value);
	}

	normalizeValue(value) {
		if (value == null && typeof value === "undefined") return null;

		return this.type.normalize(value);
	}

	async updatedValue(value) {
		const currentValue =  this.type.getValue();
		if (value != currentValue)
			this.type.setValue(value)
	}

	set readonly(readonly) {
		this.input.attr("disabled", readonly ? "" : null);
	}

	get value() {
		return this.type.getValue();
	}

	get valid() {
		return this.input.checkValidity();
	}
}
